/**************************************************************************** 
*  Running LED Sample														*
*  By Sorat Aunhawarakorn.													*
*  SEMICONDUCTOR ELECTRONIC JOURNAL.										*
*  Fisinhed version 19/12/46												*	
*****************************************************************************/
#include <AT892051.H>
#include <stdio.h>
#include "table.h"

sbit	SCLK = P1^6;									// Serial Clock
sbit	SDATA = P1^7;									// Serial Data

/************************** Declare function ********************************/

void delay(unsigned int i);
void CLKOUT(void);
void LdSER(unsigned char x,unsigned char y);

/************************** Global instance *********************************/

bit	Rflag;												// Status flag
unsigned char DBUFF[6],CBUFF[2];						// Display buffer and Serial buffer
unsigned char CBUFF_Count,BID;
unsigned int Result,BID6;
unsigned char row;
/*****************************************************************************/

void main(void)
{
//	unsigned char row;
	unsigned int i;

	SCON = 0x52;										// Set speacial function register
	TMOD = 0x20;
	TH1 = 0xFD;											// Baud Rate = 9600bps
	
	CBUFF_Count = 0;									// Initial value
	TR1 = 1;											// Start Timer1

	RI = 0;
	TI = 0;
	REN = 1;
	ES = 1;												// Enable Serial Port interrupt
	PS = 0;												// Set serial interrupts to low priority
	EA = 1;												// Enable Global interrupt
	Rflag = 0;											// Initial flag to zero

	for(;;)
	{
		for(row=0;row<7;row++)
		{
			P1 = 0x07;									// Clear line
			for(i=0;i<30;i++)							// Clear S2P Buffer
			{
				SDATA = 1;								// Set DATA pin to HIGH
				CLKOUT();
			}

			LdSER(DBUFF[0],row);						// 1st character
			LdSER(DBUFF[1],row);						// 2nd character
			LdSER(DBUFF[2],row);						// 3th character
			LdSER(DBUFF[3],row);						// 4th character
			LdSER(DBUFF[4],row);						// 5th character
			LdSER(DBUFF[5],row);						// 6th character

			if (row<7)
 		    {
				P1 = row;
				delay(50);
			}
		}
	}
}

/***************************************************************************** 
* My Routine																 *
*****************************************************************************/
void delay(unsigned int i)								// Delay Loop
{
	int j;
	for (j=0;j<i;j++);
}

void CLKOUT(void)										// Clock out P1.6
{
	SCLK = 0;
	SCLK = 1;
}

void LdSER(unsigned char x,unsigned char y)				// Load data to SER2PAR
{
	unsigned char buffer;
	unsigned int i;

	buffer = FONT_Table[x][y];

	for(i=0;i<5;i++)
	{
		SDATA = (buffer&0x10);							// Set DATA pin to HIGH
		CLKOUT();
		buffer = buffer << 1;
	}
}

/******************************************************************** 
* Interrupt Service routine											*
*********************************************************************/
static void serial_int(void) interrupt 4
{
	if (Rflag==1)
	{
	    RI=0;										// Clear RI flag
													// CBUFF[0] = Digit No.
													// CBUFF[1] = Data
		if(CBUFF_Count<2)
		{
			CBUFF[CBUFF_Count] = SBUF;
			CBUFF_Count++;

			BID = P3&0x3C;							// Get Board ID
			BID = BID>>2;							// Shift right 2
			
			if ((CBUFF[0]<(6*BID))||(CBUFF[0]>(6*BID)+6))
			{
				Rflag = 0;			
			}
		}
		
		else
		{
			if(SBUF == 'S')								// Check Stop byte (4th byte)
			{
				BID = P3&0x3C;							// Get Board ID
				BID = BID>>2;							// Shift right 2
				Result = CBUFF[0]-(BID*6);
				DBUFF[Result] = CBUFF[1]-0x20;			// Convert to Font Table

				if ((DBUFF[Result]<0x00)||(DBUFF[Result]>0x5E))	// Checking for out of FONT range
				{
					DBUFF[Result] = 'x'-0x20;	
				}
			}
				
			Rflag = 0;
			row=7;
		}
	}
	
	else
	{
		RI=0;											// Clear RI flag
		
		if(SBUF == 'A')									// Check Start Byte
		{
		    Rflag = 1;									// Set Flag
			CBUFF_Count = 0;							// Define Counter to 1
		}
		
		else if(SBUF == 0x06)
		{
			DBUFF[0] = ' '-0x20;
			DBUFF[1] = DBUFF[0];
			DBUFF[2] = DBUFF[0];
			DBUFF[3] = DBUFF[0];
			DBUFF[4] = DBUFF[0];
			DBUFF[5] = DBUFF[0];
			Rflag = 0;									// Clear Flag
			row=7;
		}
	}
}


